/*
 * File:	clipdef1.cc
 * Purpose:	Basic CLIPS function definitions
 * Author:	Julian Smart
 * Created:	1993
 * Updated:	
 * Copyright:	(c) 1993, AIAI, University of Edinburgh
 */

static const char sccsid[] = "%W% %G%";

#include "wx_hash.h"
#include "clips2c.h"

void InitializeClips(void);
void InitializeWXClips(void);
void InitializeHardy(void);
void InitializeHardyCompatibility(void);

void InitializeClips2C(int theMode)
{
  SetGenerationMode(theMode);
  
  // Delete old functions in case we're re-initializing
  TheFunctionTable.DeleteContents(TRUE);
  TheFunctionTable.Clear();
  TheFunctionTable.DeleteContents(FALSE);

  InitializeClips();

  if (theMode == MODE_PLAIN_CLIPS)
    return;

  InitializeWXClips();

  /*
   * HARDY-specific functions
   *
   */

  if (theMode == MODE_WXCLIPS)
    return;

  InitializeHardy();
  InitializeHardyCompatibility();
}

void InitializeClips(void)
{
  /*
   * Normal CLIPS
   *
   */
  AddClipsFunction(cInteger, "not", "not", cInteger);
  AddClipsFunction(cInteger, "and", "and", cInteger, cInteger);
  AddClipsFunction(cInteger, "or", "or", cInteger, cInteger);

  // String handling
  AddClipsFunction(cStringClass, "str-cat", "+", cStringClass, cStringClass);
  AddClipsFunction(cStringClass, "sym-cat", "+", cStringClass, cStringClass);
  AddClipsFunction(cStringClass, "sub-string", "substring", cStringClass, cStringClass);
  AddClipsFunction(cStringClass, "upcase", "upcase", cStringClass);
  AddClipsFunction(cStringClass, "lowcase", "lowcase", cStringClass);
  AddClipsFunction(cInteger, "str-compare", "strcmp", cStringClass, cStringClass);
  AddClipsFunction(cInteger, "str-length", "strlen", cStringClass);
  AddClipsFunction(cString, "gensym", "ClipsGensym");
  AddClipsFunction(cString, "gensym*", "ClipsGensym");
  AddClipsFunction(cVoid, "setgen", "ClipsSetGen", cInteger);

  // IO functions
  AddClipsFunction(cInteger, "open", "open", cStringClass, cStringClass, cStringClassOpt);
  AddClipsFunction(cInteger, "close", "close", cStringClassOpt);
  AddClipsFunction(cStringClass, "readline", "readline", cStringClassOpt);
  AddClipsFunction(cUnknown, "read", "read", cStringClassOpt);

  // File functions
  AddClipsFunction(cInteger, "remove", "remove", cString);
  AddClipsFunction(cInteger, "rename", "rename", cString, cString);

  // Misc functions
  AddClipsFunction(cStringClass, "type", "ClipsGetType", cUnknown);
  AddClipsFunction(cInteger, "time", "ClipsTime");

  // Numeric functions
  AddClipsFunction(cReal, "abs", "fabs", cReal);
  AddClipsFunction(cReal, "acos", "acos", cReal);
  AddClipsFunction(cReal, "acosh", "acosh", cReal);
  AddClipsFunction(cReal, "acot", "acot", cReal);
  AddClipsFunction(cReal, "acoth", "acoth", cReal);
  AddClipsFunction(cReal, "acsc", "acsc", cReal);
  AddClipsFunction(cReal, "acsch", "acsh", cReal);
  AddClipsFunction(cReal, "asec", "asec", cReal);
  AddClipsFunction(cReal, "asech", "asech", cReal);
  AddClipsFunction(cReal, "asin", "asin", cReal);
  AddClipsFunction(cReal, "asinh", "asinh", cReal);
  AddClipsFunction(cReal, "atan", "atan", cReal);
  AddClipsFunction(cReal, "atanh", "atanh", cReal);
  AddClipsFunction(cReal, "cos", "cos", cReal);
  AddClipsFunction(cReal, "cosh", "cosh", cReal);
  AddClipsFunction(cReal, "cot", "cot", cReal);
  AddClipsFunction(cReal, "coth", "coth", cReal);
  AddClipsFunction(cReal, "csc", "csc", cReal);
  AddClipsFunction(cReal, "csch", "csch", cReal);
  AddClipsFunction(cReal, "sec", "sec", cReal);
  AddClipsFunction(cReal, "sech", "sech", cReal);
  AddClipsFunction(cReal, "sin", "sin", cReal);
  AddClipsFunction(cReal, "sinh", "sinh", cReal);
  AddClipsFunction(cReal, "tan", "tan", cReal);
  AddClipsFunction(cReal, "tanh", "tanh", cReal);

  AddClipsFunction(cReal, "sqrt", "sqrt", cReal);
  AddClipsFunction(cReal, "exp", "exp", cReal);
  AddClipsFunction(cReal, "log", "log", cReal);
  AddClipsFunction(cReal, "log10", "log10", cReal);
  AddClipsFunction(cInteger, "round", "ClipsRound", cReal);
  AddClipsFunction(cReal, "mod", "mod", cReal);

  AddClipsFunction(cReal, "float", "ClipsMakeReal", cInteger);
  AddClipsFunction(cInteger, "integer", "ClipsMakeInteger", cReal);

  AddClipsFunction(cInteger, "random", "rand");
  AddClipsFunction(cVoid, "seed", "srand", cInteger);

  // List handling
  AddClipsFunction(cList, "delete", "delete", cList, cInteger, cInteger);
  AddClipsFunction(cList, "delete$", "delete", cList, cInteger, cInteger);
  AddClipsFunction(cList, "subseq", "subseq", cList, cInteger, cInteger);
  AddClipsFunction(cList, "subseq$", "subseq", cList, cInteger, cInteger);
  AddClipsFunction(cList, "explode", "explode", cStringClass);
  AddClipsFunction(cList, "explode$", "explode", cStringClass);
  AddClipsFunction(cStringClass, "implode", "implode", cList);
  AddClipsFunction(cStringClass, "implode$", "implode", cList);
  AddClipsFunction(cList, "replace", "replace", cList, cInteger, cUnknown);
  AddClipsFunction(cList, "replace$", "replace", cList, cInteger, cUnknown);
  AddClipsFunction(cList, "insert", "insert", cList, cInteger, cUnknown);
  AddClipsFunction(cList, "insert$", "insert", cList, cInteger, cUnknown);
  AddClipsFunction(cList, "first", "first", cList);
  AddClipsFunction(cList, "first$", "first", cList);
  AddClipsFunction(cList, "rest", "rest", cList);
  AddClipsFunction(cList, "rest$", "rest", cList);
}
